@@ -1,14 +1,14 @@
 #
-# Author:: Adam Jacob (<adam@opscode.com>)
-# Copyright:: Copyright (c) 2008 Opscode, Inc.
+# Author:: Adam Jacob (<adam@chef.io>)
+# Copyright:: Copyright (c) 2008-2016 Chef Software, Inc.
 # License:: Apache License, Version 2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-# 
+#
 #     http://www.apache.org/licenses/LICENSE-2.0
-# 
+#
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -16,12 +16,12 @@
 # limitations under the License.
 #
 
-$: << File.join(File.dirname(__FILE__), '..', '..', 'lib')
+$: << File.join(File.dirname(__FILE__), "..", "..", "lib")
 
-require 'spec/expectations'
-require 'mixlib/config'
-require 'tmpdir'
-require 'stringio'
+require "rspec/expectations"
+require "mixlib/config"
+require "tmpdir"
+require "stringio"
 
 class MyWorld
 end
