@@ -2,7 +2,7 @@
 # Author:: Adam Jacob (<adam@chef.io>)
 # Author:: Nuo Yan (<nuo@chef.io>)
 # Author:: Christopher Brown (<cb@chef.io>)
-# Copyright:: Copyright (c) 2008-2016 Chef Software, Inc.
+# Copyright:: Copyright (c) 2008-2018, Chef Software Inc.
 # License:: Apache License, Version 2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -18,11 +18,11 @@
 # limitations under the License.
 #
 
-require "mixlib/config/version"
-require "mixlib/config/configurable"
-require "mixlib/config/unknown_config_option_error"
-require "mixlib/config/reopened_config_context_with_configurable_error"
-require "mixlib/config/reopened_configurable_with_config_context_error"
+require_relative "config/version"
+require_relative "config/configurable"
+require_relative "config/unknown_config_option_error"
+require_relative "config/reopened_config_context_with_configurable_error"
+require_relative "config/reopened_configurable_with_config_context_error"
 
 module Mixlib
   module Config
@@ -33,11 +33,11 @@ module Mixlib
       class << base; attr_accessor :config_context_lists; end
       class << base; attr_accessor :config_context_hashes; end
       class << base; attr_accessor :config_parent; end
-      base.configuration = Hash.new
-      base.configurables = Hash.new
-      base.config_contexts = Hash.new
-      base.config_context_lists = Hash.new
-      base.config_context_hashes = Hash.new
+      base.configuration = ({})
+      base.configurables = ({})
+      base.config_contexts = ({})
+      base.config_context_lists = ({})
+      base.config_context_hashes = ({})
       base.initialize_mixlib_config
     end
 
@@ -70,7 +70,7 @@ module Mixlib
     # filename<String>:: A filename to read from
     def from_yaml(filename)
       require "yaml"
-      from_hash(YAML.load(IO.read(filename)), filename)
+      from_hash(YAML.load(IO.read(filename)))
     end
 
     # Parses valid JSON structure into Ruby
@@ -79,7 +79,7 @@ module Mixlib
     # filename<String>:: A filename to read from
     def from_json(filename)
       require "json"
-      from_hash(JSON.parse(IO.read(filename)), filename)
+      from_hash(JSON.parse(IO.read(filename)))
     end
 
     def from_toml(filename)
@@ -91,20 +91,8 @@ module Mixlib
     #
     # === Parameters
     # hash<Hash>:: A Hash containing configuration
-    def from_hash(hash, filename = "in_memory")
-      ruby_translation = []
-
-      to_dotted_hash(hash).each do |k, v|
-        if v.is_a? Array
-          ruby_translation << "#{k} #{v}"
-        elsif v.is_a? String
-          ruby_translation << "#{k} \"#{v}\""
-        else
-          ruby_translation << "#{k} #{v}"
-        end
-      end
-
-      instance_eval(ruby_translation.join("\n"), filename, 1)
+    def from_hash(hash)
+      apply_nested_hash(hash)
     end
 
     # Pass Mixlib::Config.configure() a block, and it will yield itself
@@ -152,8 +140,19 @@ module Mixlib
     # === Returns
     # <True>:: If the config option exists
     # <False>:: If the config option does not exist
-    def has_key?(key)
-      configuration.has_key?(key.to_sym)
+    def key?(key)
+      configuration.key?(key.to_sym) || config_contexts.key?(key.to_sym)
+    end
+
+    alias_method :has_key?, :key?
+
+    def is_default?(key)
+      symbol = key.to_sym
+      if configurables.key?(symbol)
+        configurables[symbol].is_default?(configuration)
+      else
+        raise ArgumentError, "config option must exist, and not be a context to check for default values"
+      end
     end
 
     # Resets a config option to its default.
@@ -166,8 +165,8 @@ module Mixlib
 
     # Resets all config options to their defaults.
     def reset
-      self.configuration = Hash.new
-      config_contexts.values.each { |config_context| config_context.reset }
+      self.configuration = ({})
+      config_contexts.values.each(&:reset)
     end
 
     # Makes a copy of any non-default values.
@@ -251,9 +250,9 @@ module Mixlib
     # === Returns
     # self
     def restore(hash)
-      self.configuration = hash.reject { |key, value| config_contexts.has_key?(key) }
+      self.configuration = hash.reject { |key, value| config_contexts.key?(key) }
       config_contexts.each do |key, config_context|
-        if hash.has_key?(key)
+        if hash.key?(key)
           config_context.restore(hash[key])
         else
           config_context.reset
@@ -261,7 +260,7 @@ module Mixlib
       end
       config_context_lists.each do |key, meta|
         meta[:values] = []
-        if hash.has_key?(key)
+        if hash.key?(key)
           hash[key].each do |val|
             context = define_context(meta[:definition_blocks])
             context.restore(val)
@@ -271,7 +270,7 @@ module Mixlib
       end
       config_context_hashes.each do |key, meta|
         meta[:values] = {}
-        if hash.has_key?(key)
+        if hash.key?(key)
           hash[key].each do |vkey, val|
             context = define_context(meta[:definition_blocks])
             context.restore(val)
@@ -290,7 +289,7 @@ module Mixlib
     # self
     def merge!(hash)
       hash.each do |key, value|
-        if config_contexts.has_key?(key)
+        if config_contexts.key?(key)
           # Grab the config context and let internal_get cache it if so desired
           config_contexts[key].restore(value)
         else
@@ -370,9 +369,10 @@ module Mixlib
     # The value of the config option.
     def configurable(symbol, &block)
       unless configurables[symbol]
-        if config_contexts.has_key?(symbol)
+        if config_contexts.key?(symbol)
           raise ReopenedConfigContextWithConfigurableError, "Cannot redefine config_context #{symbol} as a configurable value"
         end
+
         configurables[symbol] = Configurable.new(symbol)
         define_attr_accessor_methods(symbol)
       end
@@ -398,11 +398,11 @@ module Mixlib
     # block<Block>: a block that will be run in the context of this new config
     # class.
     def config_context(symbol, &block)
-      if configurables.has_key?(symbol)
+      if configurables.key?(symbol)
         raise ReopenedConfigurableWithConfigContextError, "Cannot redefine config value #{symbol} with a config context"
       end
 
-      if config_contexts.has_key?(symbol)
+      if config_contexts.key?(symbol)
         context = config_contexts[symbol]
       else
         context = Class.new
@@ -436,11 +436,11 @@ module Mixlib
     # block<Block>: a block that will be run in the context of this new config
     # class.
     def config_context_list(plural_symbol, singular_symbol, &block)
-      if configurables.has_key?(symbol)
+      if configurables.key?(plural_symbol)
         raise ReopenedConfigurableWithConfigContextError, "Cannot redefine config value #{plural_symbol} with a config context"
       end
 
-      unless config_context_lists.has_key?(plural_symbol)
+      unless config_context_lists.key?(plural_symbol)
         config_context_lists[plural_symbol] = {
           definition_blocks: [],
           values: [],
@@ -468,11 +468,11 @@ module Mixlib
     # block<Block>: a block that will be run in the context of this new config
     # class.
     def config_context_hash(plural_symbol, singular_symbol, &block)
-      if configurables.has_key?(symbol)
+      if configurables.key?(plural_symbol)
         raise ReopenedConfigurableWithConfigContextError, "Cannot redefine config value #{plural_symbol} with a config context"
       end
 
-      unless config_context_hashes.has_key?(plural_symbol)
+      unless config_context_hashes.key?(plural_symbol)
         config_context_hashes[plural_symbol] = {
           definition_blocks: [],
           values: {},
@@ -532,9 +532,10 @@ module Mixlib
     # <ArgumentError>:: if value is set to something other than true, false, or :warn
     #
     def config_strict_mode=(value)
-      if ![ true, false, :warn, nil ].include?(value)
+      unless [ true, false, :warn, nil ].include?(value)
         raise ArgumentError, "config_strict_mode must be true, false, nil or :warn"
       end
+
       @config_strict_mode = value
     end
 
@@ -556,6 +557,28 @@ module Mixlib
       internal_get_or_set(method_symbol, *args)
     end
 
+    protected
+
+    # Given a (nested) Hash, apply it to the config object and any contexts.
+    #
+    # This is preferable to converting it to the string representation with
+    # the #to_dotted_hash method above.
+    #
+    # === Parameters
+    # hash<Hash>:: The hash to apply to the config oject
+    def apply_nested_hash(hash)
+      hash.each do |k, v|
+        if v.is_a? Hash
+          # If loading from hash, and we reference a context that doesn't exist
+          # and warning/strict is off, we need to create the config context that we expected to be here.
+          context = internal_get(k.to_sym) || config_context(k.to_sym)
+          context.apply_nested_hash(v)
+        else
+          internal_set(k.to_sym, v)
+        end
+      end
+    end
+
     private
 
     # Given a (nested) Hash, turn it into a single top-level hash using dots as
@@ -586,9 +609,9 @@ module Mixlib
     # value<Object>:: Value to be set in config hash
     #
     def internal_set(symbol, value)
-      if configurables.has_key?(symbol)
+      if configurables.key?(symbol)
         configurables[symbol].set(configuration, value)
-      elsif config_contexts.has_key?(symbol)
+      elsif config_contexts.key?(symbol)
         config_contexts[symbol].restore(value.to_hash)
       else
         if config_strict_mode == :warn
@@ -601,13 +624,13 @@ module Mixlib
     end
 
     def internal_get(symbol)
-      if configurables.has_key?(symbol)
+      if configurables.key?(symbol)
         configurables[symbol].get(configuration)
-      elsif config_contexts.has_key?(symbol)
+      elsif config_contexts.key?(symbol)
         config_contexts[symbol]
-      elsif config_context_lists.has_key?(symbol)
+      elsif config_context_lists.key?(symbol)
         config_context_lists[symbol]
-      elsif config_context_hashes.has_key?(symbol)
+      elsif config_context_hashes.key?(symbol)
         config_context_hashes[symbol]
       else
         if config_strict_mode == :warn
@@ -684,7 +707,7 @@ module Mixlib
       # Adds a single new context to the list
       meta.send :define_method, singular_symbol do |key, &block|
         context_hash_details = internal_get(plural_symbol)
-        context = if context_hash_details[:values].has_key? key
+        context = if context_hash_details[:values].key? key
                     context_hash_details[:values][key]
                   else
                     new_context = define_context(context_hash_details[:definition_blocks])
