@@ -1,6 +1,6 @@
 #
 # Author:: Adam Jacob (<adam@chef.io>)
-# Copyright:: Copyright (c) 2008-2016 Chef Software, Inc.
+# Copyright:: Copyright (c) 2008-2018, Chef Software Inc.
 # License:: Apache License, Version 2.0
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -115,6 +115,26 @@ describe Mixlib::Config do
     it "raises an error when you set an arbitrary config option with [:y] = 10" do
       expect(lambda { StrictClass[:y] = 10 }).to raise_error(Mixlib::Config::UnknownConfigOptionError, "Cannot set unsupported config value y.")
     end
+
+    it "does not break config_context_list" do
+      expect(lambda do
+        StrictClass.class_eval do
+          config_context_list(:lists, :list) do
+            default :y, 20
+          end
+        end
+      end).not_to raise_error
+    end
+
+    it "does not break config_context_hash" do
+      expect(lambda do
+        StrictClass.class_eval do
+          config_context_hash(:hashes, :hash) do
+            default :z, 20
+          end
+        end
+      end).not_to raise_error
+    end
   end
 
   describe "when a block has been used to set config values" do
@@ -122,7 +142,7 @@ describe Mixlib::Config do
       ConfigIt.configure { |c| c[:cookbook_path] = "monkey_rabbit"; c[:otherthing] = "boo" }
     end
 
-    { :cookbook_path => "monkey_rabbit", :otherthing => "boo" }.each do |k, v|
+    { cookbook_path: "monkey_rabbit", otherthing: "boo" }.each do |k, v|
       it "allows you to retrieve the config value for #{k} via []" do
         expect(ConfigIt[k]).to eql(v)
       end
@@ -137,9 +157,15 @@ describe Mixlib::Config do
   end
 
   it "returns true or false with has_key?" do
-    expect(ConfigIt.has_key?(:monkey)).to be false
+    expect(ConfigIt.key?(:monkey)).to be false
     ConfigIt[:monkey] = "gotcha"
-    expect(ConfigIt.has_key?(:monkey)).to be true
+    expect(ConfigIt.key?(:monkey)).to be true
+  end
+
+  it "returns true or false with key?" do
+    expect(ConfigIt.key?(:monkey2)).to be false
+    ConfigIt[:monkey2] = "gotcha"
+    expect(ConfigIt.key?(:monkey2)).to be true
   end
 
   describe "when a class method override writer exists" do
@@ -209,6 +235,23 @@ describe Mixlib::Config do
       expect(@klass.respond_to?("z=".to_sym)).to be false
     end
 
+    it "returns true for is_default? for a default value" do
+      expect(@klass[:a]).to eql(1)
+      expect(@klass.is_default?(:a)).to be true
+    end
+
+    it "returns true for is_default? for an overwritten default value" do
+      @klass[:a] = 1
+      expect(@klass[:a]).to eql(1)
+      expect(@klass.is_default?(:a)).to be true
+    end
+
+    it "returns false for is_default? for a value that is not the default" do
+      @klass[:a] = 2
+      expect(@klass[:a]).to eql(2)
+      expect(@klass.is_default?(:a)).to be false
+    end
+
     describe "and extra methods have been dumped into Object" do
       class NopeError < StandardError
       end
@@ -281,12 +324,12 @@ describe Mixlib::Config do
     end
 
     it "save with include_defaults should save all defaults" do
-      expect(@klass.save(true)).to eql({ :attr => 4 })
+      expect(@klass.save(true)).to eql({ attr: 4 })
     end
 
     it "saves the new value if it gets set" do
       @klass.attr 5
-      expect((saved = @klass.save)).to eql({ :attr => 5 })
+      expect((saved = @klass.save)).to eql({ attr: 5 })
       @klass.reset
       expect(@klass.attr).to eql(4)
       @klass.restore(saved)
@@ -295,11 +338,11 @@ describe Mixlib::Config do
 
     it "saves the new value even if it is set to its default value" do
       @klass.attr 4
-      expect((saved = @klass.save)).to eql({ :attr => 4 })
+      expect((saved = @klass.save)).to eql({ attr: 4 })
       @klass.reset
       expect(@klass.save).to eql({})
       @klass.restore(saved)
-      expect(@klass.save).to eql({ :attr => 4 })
+      expect(@klass.save).to eql({ attr: 4 })
     end
   end
 
@@ -349,12 +392,12 @@ describe Mixlib::Config do
     end
 
     it "save with include_defaults should save all defaults" do
-      expect(@klass.save(true)).to eql({ :attr => 8, :x => 4 })
+      expect(@klass.save(true)).to eql({ attr: 8, x: 4 })
     end
 
     it "saves the new value if it gets set" do
       @klass.attr 5
-      expect((saved = @klass.save)).to eql({ :attr => 5 })
+      expect((saved = @klass.save)).to eql({ attr: 5 })
       @klass.reset
       expect(@klass.attr).to eql(8)
       @klass.restore(saved)
@@ -363,11 +406,11 @@ describe Mixlib::Config do
 
     it "saves the new value even if it is set to its default value" do
       @klass.attr 8
-      expect((saved = @klass.save)).to eql({ :attr => 8 })
+      expect((saved = @klass.save)).to eql({ attr: 8 })
       @klass.reset
       expect(@klass.save).to eql({})
       @klass.restore(saved)
-      expect(@klass.save).to eql({ :attr => 8 })
+      expect(@klass.save).to eql({ attr: 8 })
     end
   end
 
@@ -390,12 +433,12 @@ describe Mixlib::Config do
     end
 
     it "save with include_defaults should save all defaults" do
-      expect(@klass.save(true)).to eql({ :attr => [] })
+      expect(@klass.save(true)).to eql({ attr: [] })
     end
 
     it "saves the new value if it gets set" do
       @klass.attr << "x"
-      expect((saved = @klass.save)).to eql({ :attr => [ "x" ] })
+      expect((saved = @klass.save)).to eql({ attr: [ "x" ] })
       @klass.reset
       expect(@klass.attr).to eql([])
       @klass.restore(saved)
@@ -404,11 +447,11 @@ describe Mixlib::Config do
 
     it "saves the new value even if it is set to its default value" do
       @klass.attr = []
-      expect((saved = @klass.save)).to eql({ :attr => [] })
+      expect((saved = @klass.save)).to eql({ attr: [] })
       @klass.reset
       expect(@klass.save).to eql({})
       @klass.restore(saved)
-      expect(@klass.save).to eql({ :attr => [] })
+      expect(@klass.save).to eql({ attr: [] })
     end
   end
 
@@ -431,25 +474,25 @@ describe Mixlib::Config do
     end
 
     it "save with include_defaults should save all defaults" do
-      expect(@klass.save(true)).to eql({ :attr => {} })
+      expect(@klass.save(true)).to eql({ attr: {} })
     end
 
     it "saves the new value if it gets set" do
       @klass.attr[:hi] = "lo"
-      expect((saved = @klass.save)).to eql({ :attr => { :hi => "lo" } })
+      expect((saved = @klass.save)).to eql({ attr: { hi: "lo" } })
       @klass.reset
       expect(@klass.attr).to eql({})
       @klass.restore(saved)
-      expect(@klass.save).to eql({ :attr => { :hi => "lo" } })
+      expect(@klass.save).to eql({ attr: { hi: "lo" } })
     end
 
     it "saves the new value even if it is set to its default value" do
       @klass.attr = {}
-      expect((saved = @klass.save)).to eql({ :attr => {} })
+      expect((saved = @klass.save)).to eql({ attr: {} })
       @klass.reset
       expect(@klass.save).to eql({})
       @klass.restore(saved)
-      expect(@klass.save).to eql({ :attr => {} })
+      expect(@klass.save).to eql({ attr: {} })
     end
   end
 
@@ -472,12 +515,12 @@ describe Mixlib::Config do
     end
 
     it "save with include_defaults should save all defaults" do
-      expect(@klass.save(true)).to eql({ :attr => "hello" })
+      expect(@klass.save(true)).to eql({ attr: "hello" })
     end
 
     it "saves the new value if it gets set" do
       @klass.attr << " world"
-      expect((saved = @klass.save)).to eql({ :attr => "hello world" })
+      expect((saved = @klass.save)).to eql({ attr: "hello world" })
       @klass.reset
       expect(@klass.attr).to eql("hello")
       @klass.restore(saved)
@@ -486,11 +529,11 @@ describe Mixlib::Config do
 
     it "saves the new value even if it is set to its default value" do
       @klass.attr "hello world"
-      expect((saved = @klass.save)).to eql({ :attr => "hello world" })
+      expect((saved = @klass.save)).to eql({ attr: "hello world" })
       @klass.reset
       expect(@klass.save).to eql({})
       @klass.restore(saved)
-      expect(@klass.save).to eql({ :attr => "hello world" })
+      expect(@klass.save).to eql({ attr: "hello world" })
     end
   end
 
@@ -534,12 +577,12 @@ describe Mixlib::Config do
     end
 
     it "save with include_defaults should save all defaults" do
-      expect(@klass.save(true)).to eql({ :attr => 4 })
+      expect(@klass.save(true)).to eql({ attr: 4 })
     end
 
     it "saves the new value if it gets set" do
       @klass.attr 5
-      expect((saved = @klass.save)).to eql({ :attr => 5 })
+      expect((saved = @klass.save)).to eql({ attr: 5 })
       @klass.reset
       expect(@klass.attr).to eql(4)
       @klass.restore(saved)
@@ -548,11 +591,11 @@ describe Mixlib::Config do
 
     it "saves the new value even if it is set to its default value" do
       @klass.attr 4
-      expect((saved = @klass.save)).to eql({ :attr => 4 })
+      expect((saved = @klass.save)).to eql({ attr: 4 })
       @klass.reset
       expect(@klass.save).to eql({})
       @klass.restore(saved)
-      expect(@klass.save).to eql({ :attr => 4 })
+      expect(@klass.save).to eql({ attr: 4 })
     end
   end
 
@@ -611,12 +654,12 @@ describe Mixlib::Config do
     end
 
     it "save with include_defaults should save all defaults" do
-      expect(@klass.save(true)).to eql({ :attr => 4 })
+      expect(@klass.save(true)).to eql({ attr: 4 })
     end
 
     it "saves the new value if it gets set" do
       @klass.attr 5
-      expect((saved = @klass.save)).to eql({ :attr => 10 })
+      expect((saved = @klass.save)).to eql({ attr: 10 })
       @klass.reset
       expect(@klass.attr).to eql(4)
       @klass.restore(saved)
@@ -625,11 +668,11 @@ describe Mixlib::Config do
 
     it "saves the new value even if it is set to its default value" do
       @klass.attr 4
-      expect((saved = @klass.save)).to eql({ :attr => 8 })
+      expect((saved = @klass.save)).to eql({ attr: 8 })
       @klass.reset
       expect(@klass.save).to eql({})
       @klass.restore(saved)
-      expect(@klass.save).to eql({ :attr => 8 })
+      expect(@klass.save).to eql({ attr: 8 })
     end
   end
 
@@ -685,12 +728,12 @@ describe Mixlib::Config do
     end
 
     it "save with include_defaults should save all defaults" do
-      expect(@klass.save(true)).to eql({ :attr => 4 })
+      expect(@klass.save(true)).to eql({ attr: 4 })
     end
 
     it "saves the new value if it gets set" do
       @klass.attr 5
-      expect((saved = @klass.save)).to eql({ :attr => 10 })
+      expect((saved = @klass.save)).to eql({ attr: 10 })
       @klass.reset
       expect(@klass.attr).to eql(4)
       @klass.restore(saved)
@@ -699,11 +742,11 @@ describe Mixlib::Config do
 
     it "saves the new value even if it is set to its default value" do
       @klass.attr 2
-      expect((saved = @klass.save)).to eql({ :attr => 4 })
+      expect((saved = @klass.save)).to eql({ attr: 4 })
       @klass.reset
       expect(@klass.save).to eql({})
       @klass.restore(saved)
-      expect(@klass.save).to eql({ :attr => 4 })
+      expect(@klass.save).to eql({ attr: 4 })
     end
   end
 
@@ -735,12 +778,12 @@ describe Mixlib::Config do
     end
 
     it "setting the entire context to a hash with default value overridden sets the value" do
-      @klass.blah = { :x => 10 }
+      @klass.blah = { x: 10 }
       expect(@klass.blah.x).to eql(10)
     end
 
     it "setting the entire context to a hash sets non-default values" do
-      @klass.blah = { :y => 10 }
+      @klass.blah = { y: 10 }
       expect(@klass.blah.x).to eql(5)
       expect(@klass.blah.y).to eql(10)
     end
@@ -748,7 +791,7 @@ describe Mixlib::Config do
     it "setting the entire context to a hash deletes any non-default values and resets default values" do
       @klass.blah.x = 10
       @klass.blah.y = 10
-      @klass.blah = { :z => 10 }
+      @klass.blah = { z: 10 }
       expect(@klass.blah.x).to eql(5)
       expect(@klass.blah.y).to be_nil
       expect(@klass.blah.z).to eql(10)
@@ -784,17 +827,17 @@ describe Mixlib::Config do
     end
 
     it "save with include_defaults should save all defaults" do
-      expect(@klass.save(true)).to eql({ :blah => { :x => 5 } })
+      expect(@klass.save(true)).to eql({ blah: { x: 5 } })
     end
 
     it "saves any new values that are set in the context" do
       @klass.blah.x = 10
-      expect((saved = @klass.save)).to eql({ :blah => { :x => 10 } })
+      expect((saved = @klass.save)).to eql({ blah: { x: 10 } })
       @klass.reset
       expect(@klass.blah.x).to eql(5)
       @klass.restore(saved)
       expect(@klass.blah.x).to eql(10)
-      expect(@klass.save).to eql({ :blah => { :x => 10 } })
+      expect(@klass.save).to eql({ blah: { x: 10 } })
     end
 
     # this tests existing (somewhat bizzare) behavior of mixlib-config where testing to
@@ -802,13 +845,13 @@ describe Mixlib::Config do
     # retrieve the default value if it was set.  the code in chef/chef which merges
     # knife config values into cli values will be sensitive to this behavior.
     it "defaults values do not show up when querying with #has_key?" do
-      expect(@klass.blah.has_key?(:x)).to be false
+      expect(@klass.blah.key?(:x)).to be false
       expect(@klass.blah.x).to be 5
     end
 
     it "if we assign the values, they show up when querying with #has_key?" do
       @klass.blah.x = 5
-      expect(@klass.blah.has_key?(:x)).to be true
+      expect(@klass.blah.key?(:x)).to be true
     end
   end
 
@@ -860,37 +903,37 @@ describe Mixlib::Config do
     end
 
     it "save with include_defaults should save all defaults" do
-      expect(@klass.save(true)).to eql({ :blah => { :yarr => { :x => 5, :y => 6 } } })
+      expect(@klass.save(true)).to eql({ blah: { yarr: { x: 5, y: 6 } } })
     end
 
     it "saves any new values that are set in the context" do
       @klass.blah.yarr.x = 10
       @klass.blah.yarr.y = 11
-      expect((saved = @klass.save)).to eql({ :blah => { :yarr => { :x => 10, :y => 11 } } })
+      expect((saved = @klass.save)).to eql({ blah: { yarr: { x: 10, y: 11 } } })
       @klass.reset
       expect(@klass.blah.yarr.x).to eql(5)
       expect(@klass.blah.yarr.y).to eql(6)
       @klass.restore(saved)
       expect(@klass.blah.yarr.x).to eql(10)
       expect(@klass.blah.yarr.y).to eql(11)
-      expect(@klass.save).to eql({ :blah => { :yarr => { :x => 10, :y => 11 } } })
+      expect(@klass.save).to eql({ blah: { yarr: { x: 10, y: 11 } } })
     end
 
     it "restores defaults not included in saved data" do
-      @klass.restore( :blah => { :yarr => { :x => 10 } } )
+      @klass.restore( blah: { yarr: { x: 10 } } )
       expect(@klass.blah.yarr.x).to eql(10)
       expect(@klass.blah.yarr.y).to eql(6)
     end
 
     it "removes added properties not included in saved state" do
       @klass.blah.yarr.z = 12
-      @klass.restore( :blah => { :yarr => { :x => 10 } } )
+      @klass.restore( blah: { yarr: { x: 10 } } )
       expect(@klass.blah.yarr.x).to eql(10)
       expect(@klass.blah.yarr.z).to eql(nil)
     end
 
     it "can set a config context from another context" do
-      @klass.blah.blyme = { :x => 7 }
+      @klass.blah.blyme = { x: 7 }
       blyme = @klass.blah.blyme
       @klass.blah.yarr.x = 12
       @klass.blah.yarr = blyme
@@ -909,12 +952,20 @@ describe Mixlib::Config do
       end
     end
 
+    it "has_key? finds the subcontext" do
+      expect(@klass.key?(:blah)).to be true
+    end
+
+    it "key? finds the subcontext" do
+      expect(@klass.key?(:blah)).to be true
+    end
+
     it "save does not save the hash for the config_context" do
       expect(@klass.save).to eql({})
     end
 
     it "save with defaults saves the hash for the config_context" do
-      expect(@klass.save(true)).to eql({ :blah => {} })
+      expect(@klass.save(true)).to eql({ blah: {} })
     end
   end
 
@@ -932,7 +983,7 @@ describe Mixlib::Config do
     end
 
     it "save with defaults saves the hash for the config_context" do
-      expect(@klass.save(true)).to eql({ :blah => {} })
+      expect(@klass.save(true)).to eql({ blah: {} })
     end
   end
 
@@ -1148,13 +1199,13 @@ describe Mixlib::Config do
 
   describe ".from_yaml" do
     let(:yaml) do
-      <<-EOH
----
-foo:
-  - bar
-  - baz
-  - matazz
-alpha: beta
+      <<~EOH
+        ---
+        foo:
+          - bar
+          - baz
+          - matazz
+        alpha: beta
       EOH
     end
 
@@ -1174,15 +1225,15 @@ alpha: beta
 
   describe ".from_json" do
     let(:json) do
-      <<-EOH
-{
-  "foo": [
-    "bar",
-    "baz",
-    "matazz"
-  ],
-  "alpha": "beta"
-}
+      <<~EOH
+        {
+          "foo": [
+            "bar",
+            "baz",
+            "matazz"
+          ],
+          "alpha": "beta"
+        }
       EOH
     end
 
@@ -1202,9 +1253,9 @@ alpha: beta
 
   describe ".from_toml" do
     let(:toml) do
-      <<-EOH
-foo = ["bar", "baz", "matazz"]
-alpha = "beta"
+      <<~EOH
+        foo = ["bar", "baz", "matazz"]
+        alpha = "beta"
       EOH
     end
 
@@ -1223,17 +1274,45 @@ alpha = "beta"
   end
 
   describe ".from_hash" do
-    let(:hash) do
-      {
-        "alpha" => "beta",
-        "foo" => %w{ bar baz matazz},
-      }
+    context "when contexts in the hash are defined" do
+      let(:hash) do
+        {
+          "alpha" => "beta",
+          gamma: "delta",
+          "foo" => %w{ bar baz matazz},
+          "bar" => { "baz" => { "fizz" => "buzz" } },
+          "windows_path" => 'C:\Windows Has Awful\Paths',
+        }
+      end
+      it "configures the config object from a hash" do
+        ConfigIt.config_context :bar do
+          config_context :baz do
+            default :fizz, "quux"
+          end
+        end
+        ConfigIt.from_hash(hash)
+        expect(ConfigIt.foo).to eql(%w{ bar baz matazz })
+        expect(ConfigIt.alpha).to eql("beta")
+        expect(ConfigIt.gamma).to eql("delta")
+        expect(ConfigIt[:bar][:baz][:fizz]).to eql("buzz")
+        expect(ConfigIt.windows_path).to eql('C:\Windows Has Awful\Paths')
+      end
     end
-
-    it "translates the Hash into method-style" do
-      ConfigIt.from_hash(hash)
-      expect(ConfigIt.foo).to eql(%w{ bar baz matazz })
-      expect(ConfigIt.alpha).to eql("beta")
+    context "when contexts in the hash are undefined and strict disabled" do
+      before do
+        ConfigIt.strict_mode = true
+      end
+      let(:hash) do
+        {
+          "brrr" => { "baz" => { "fizz" => "buzz" } },
+          "windows_path" => 'C:\Windows Has Awful\Paths',
+        }
+      end
+      it "configures the config object, creating contexts as needed" do
+        ConfigIt.from_hash(hash)
+        expect(ConfigIt[:brrr][:baz][:fizz]).to eql("buzz")
+        expect(ConfigIt.windows_path).to eql('C:\Windows Has Awful\Paths')
+      end
     end
   end
 end
