@@ -9,7 +9,7 @@ class ConfigIt
 end
 
 RSpec.configure do |config|
-  config.filter_run :focus => true
+  config.filter_run focus: true
   config.run_all_when_everything_filtered = true
   config.warnings = true
 end
